//=============================================================================
// Change_TP_Custom_MZ.js
//   Last update: 2024/09/22
//=============================================================================
// 更新っぽいもの
// 
// 2024/09/22 [v0.1.0]
// 初版

/*:
 * @target MZ
 * 
 * @plugindesc
 * [v0.1.0] Change!! TP Custom MZ.
 * 
 * @author Diamond
 *
 * @param TP Init Value
 * @text ????
 * @default 0
 * @type number
 * @desc
 * English is not supported.
 * Please Use Japanese!!
 * 
 * @param TP Carry Over
 * @text ????
 * @default 0
 * @type number
 * @desc
 * English is not supported.
 * Please Use Japanese!!
 * 
 * @help
 * Diamond use ruby, not use Java Script!!
 * Please use Ruby!! and RPG Tkool XP/VX/VX Ace.
 * 
 * @url https://diamond.iza-yoi.net/
 */

/*:ja
 * @target MZ
 * 
 * @plugindesc
 * [v0.1.0] 真（チェンジ！！）TP改 MZ
 * 
 * @author 貪藻矢射妥←
 * 
 * @param TP Init Value
 * @text TP初期値
 * @default 0
 * @type number
 * @desc
 * TP初期値を指定します。
 * 0 : 0始まり / 1以降 : 乱数始まり
 * 
 * @param TP Carry Over
 * @text TP持ち越し設定
 * @default 0
 * @type number
 * @desc
 * TP持ち越し設定を指定します。
 * 0 : Actor能力依存 / 1 : 一律持ち越さず / 2 : 一律持ち越す
 *
 * @help
 * TPの設定についてカスタマイズします。
 * 
 * また、戦闘不能時にTPゲージがMaxになるのはおかしいのでは？と思い
 * TPを0にするよに変更しています。
 *
 * Java ScriptよりもRubyを使いたい！ by 貪藻矢射妥←]
 * 
 * @url https://diamond.iza-yoi.net/
 */

var OuterFlgs = OuterFlgs || {};

(() => {

OuterFlgs.imported = OuterFlgs.imported || {};
OuterFlgs.imported['TP_Custom'] = true;

var pluginParams = PluginManager.parameters('Change_TP_Custom_MZ');
var changeTPCustomInitValue = Number(pluginParams['TP Init Value'] || 0);
var changeTPCustomCarryOver = Number(pluginParams['TP Carry Over'] || 0);

Game_BattlerBase.TP_CARRY_OVER_AT_ACTOR    = 0;
Game_BattlerBase.TP_CARRY_OVER_IS_NONE     = 1;
Game_BattlerBase.TP_CARRY_OVER_IS_MUST     = 2;

//-----------------------------------------------------------------------------
// Game_Battler

Game_Battler.prototype.gainSilentTp = function(value) {
    if (!this.isDead()) {
        // アクターが死ななかった場合、TP増加(被ダメージ時)
        this.setTp(this.tp + value);
    } else {
        // アクターが死んだ場合、TPは0に戻す
        this.clearTp();
    }
};

Game_Battler.prototype.initTp = function() {
    if (changeTPCustomInitValue === 0) {
        // TPが0始まり固定
        this.setTp(0);
    } else {
        // TPの初期値を乱数で決定
        this.setTp(Math.randomInt(changeTPCustomInitValue));
    }
};

Game_Battler.prototype.onBattleStart = function(advantageous) {
    this.setActionState("undecided");
    this.clearMotion();
    this.initTpbChargeTime(advantageous);
    this.initTpbTurn();
    
    switch (changeTPCustomCarryOver){
    case Game_BattlerBase.TP_CARRY_OVER_AT_ACTOR:
        // アクターの能力如何でTP持ち越しを決める
        if (!this.isPreserveTp()) {
            this.initTp();
        }
        break;
    case Game_BattlerBase.TP_CARRY_OVER_IS_MUST:
        // 一律TPを持ち越す
        // TPの初期化を実施しない
        break;
    case Game_BattlerBase.TP_CARRY_OVER_IS_NONE:
    default:
        // 一律TPを持ち越さない
        this.initTp();
        break;
    }
};

Game_Battler.prototype.onBattleEnd = function() {
    this.clearResult();
    this.removeBattleStates();
    this.removeAllBuffs();
    this.clearActions();
    
    switch (changeTPCustomCarryOver){
    case Game_BattlerBase.TP_CARRY_OVER_AT_ACTOR:
        // アクターの能力如何でTP持ち越しを決める
        if (!this.isPreserveTp()) {
            this.clearTp();
        }
        break;
    case Game_BattlerBase.TP_CARRY_OVER_IS_MUST:
        // 一律TPを持ち越す
        // TPの初期化を実施しない
        break;
    case Game_BattlerBase.TP_CARRY_OVER_IS_NONE:
    default:
        // 一律TPを持ち越さない
        this.clearTp();
        break;
    }
    
    this.appear();
};

//-----------------------------------------------------------------------------
// Sprite_Gauge

Sprite_Gauge.prototype.isValid = function() {
    if (this._battler) {
        if (this._statusType === "tp") {
            switch (changeTPCustomCarryOver){
            case Game_BattlerBase.TP_CARRY_OVER_AT_ACTOR:
                // 基本的にはアクターの能力如何で戻り値を決める
                // ただし、戦闘中は一律true
                if (!this._battler.isPreserveTp()) {
                    return $gameParty.inBattle();
                } else {
                    return true;
                }
                break;
            case Game_BattlerBase.TP_CARRY_OVER_IS_MUST:
                // 一律true
                return true;
                break;
            case Game_BattlerBase.TP_CARRY_OVER_IS_NONE:
            default:
                // 一律false
                return false;
                break;
            }
        } else {
            return true;
        }
    }
    return false;
};

})();
